@@ -111,6 +111,7 @@ madlib.lm <- function (formula, data, na.action = NULL,
     r.has.intercept <- params$has.intercept # do we have an intercept
     ## r.ind.vars <- gsub("\"", "", params$ind.vars)
     r.ind.vars <- params$ind.vars
+    r.origin.ind <- params$origin.ind
     r.ind.str <- params$ind.str
     r.col.name <- gsub("\"", "", data@.col.name)
     r.appear <- data@.appear.name
@@ -130,6 +131,7 @@ madlib.lm <- function (formula, data, na.action = NULL,
         rst[[i]]$grp.expr <- r.grp.expr
         rst[[i]]$has.intercept <- r.has.intercept
         rst[[i]]$ind.vars <- r.ind.vars
+        rst[[i]]$origin.ind <- r.origin.ind
         rst[[i]]$ind.str <- r.ind.str
         rst[[i]]$col.name <- r.col.name
         rst[[i]]$appear <- r.appear
@@ -138,6 +140,7 @@ madlib.lm <- function (formula, data, na.action = NULL,
         rst[[i]]$dummy.expr <- r.dummy.expr
         rst[[i]]$model <- model
         rst[[i]]$terms <- params$terms
+        rst[[i]]$factor.ref <- data@.factor.ref
 
         if (length(r.grp.cols) != 0) {
             ## cond <- Reduce(function(l, r) l & r,
@@ -216,10 +219,13 @@ print.lm.madlib.grps <- function (x,
     else
         rows <- x[[1]]$ind.vars
     rows <- gsub("\"", "", rows)
+    rows <- gsub("::[\\w\\s]+", "", rows, perl = T)
     for (i in seq_len(length(x[[1]]$col.name))) 
         if (x[[1]]$col.name[i] != x[[1]]$appear[i])
             rows <- gsub(x[[1]]$col.name[i], x[[1]]$appear[i], rows)
-    rows <- gsub("\\((.*)\\)\\[(\\d+)\\]", "\\1[\\2]", rows)
+    rows <- gsub("\\(([^\\[\\]]*)\\)\\[(\\d+)\\]", "\\1[\\2]", rows)
+    rows <- .reverse.consistent.func(rows)
+    rows <- gsub("\\s", "", rows)
     ind.width <- .max.width(rows)
 
     cat("\nMADlib Linear Regression Result\n")
@@ -240,38 +246,12 @@ print.lm.madlib.grps <- function (x,
         }
 
         cat("Coefficients:\n")
-        coef <- format(x[[i]]$coef, digits = digits)
-        std.err <- format(x[[i]]$std_err, digits = digits)
-        t.stats <- format(x[[i]]$t_stats, digits = digits)
-
-        stars <- rep("", length(x[[i]]$p_values))
-        for (j in seq(length(x[[i]]$p_values))) {
-            if (is.na(x[[i]]$p_values[j]) || is.nan(x[[i]]$p_values[j])) {
-                stars[j] <- " "
-                next
-            }
-            if (x[[i]]$p_values[j] < 0.001)
-                stars[j] <- "***"
-            else if (x[[i]]$p_values[j] < 0.01)
-                stars[j] <- "**"
-            else if (x[[i]]$p_values[j] < 0.05)
-                stars[j] <- "*"
-            else if (x[[i]]$p_values[j] < 0.1)
-                stars[j] <- "."
-            else
-                stars[j] <- " "
-        }
-        p.values <- paste(format(x[[i]]$p_values, digits = digits),
-                          stars)
-        output <- data.frame(cbind(Estimate = coef,
-                                   `Std. Error` = std.err,
-                                   `t value` = t.stats,
-                                   `Pr(>|t|)` = p.values),
-                             row.names = rows, check.names = FALSE)
-        print(format(output, justify = "left"))
-
-        cat("---\n")
-        cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n\n")
+        printCoefmat(data.frame(cbind(Estimate = x[[i]]$coef,
+                                      `Std. Error` = x[[i]]$std_err,
+                                      `t value` = x[[i]]$t_stats,
+                                      `Pr(>|t|)` = x[[i]]$p_values),
+                                row.names = rows, check.names = FALSE),
+                 digits = digits, signif.stars = TRUE)
         cat("R-squared:", x[[i]]$r2, "\n")
         cat("Condition Number:", x[[i]]$condition_no, "\n")
 
@@ -305,10 +285,13 @@ print.lm.madlib <- function (x,
     else
         rows <- x$ind.vars
     rows <- gsub("\"", "", rows)
+    rows <- gsub("::[\\w\\s]+", "", rows, perl = T)
     for (i in seq_len(length(x$col.name))) 
         if (x$col.name[i] != x$appear[i])
             rows <- gsub(x$col.name[i], x$appear[i], rows)
-    rows <- gsub("\\((.*)\\)\\[(\\d+)\\]", "\\1[\\2]", rows)
+    rows <- gsub("\\(([^\\[\\]]*)\\)\\[(\\d+)\\]", "\\1[\\2]", rows)
+    rows <- .reverse.consistent.func(rows)
+    rows <- gsub("\\s", "", rows)
     ind.width <- .max.width(rows)
 
     cat("\nMADlib Linear Regression Result\n")
@@ -324,38 +307,13 @@ print.lm.madlib <- function (x,
     }
     
     cat("Coefficients:\n")
-    coef <- format(x$coef, digits = digits)
-    std.err <- format(x$std_err, digits = digits)
-    t.stats <- format(x$t_stats, digits = digits)
+    printCoefmat(data.frame(cbind(Estimate = x$coef,
+                                  `Std. Error` = x$std_err,
+                                  `t value` = x$t_stats,
+                                  `Pr(>|t|)` = x$p_values),
+                            row.names = rows, check.names = FALSE),
+                 digits = digits, signif.stars = TRUE)
     
-    stars <- rep("", length(x$p_values))
-    for (j in seq(length(x$p_values))) {
-        if (is.na(x$p_values[j]) || is.nan(x$p_values[j])) {
-            stars[j] <- " "
-            next
-        }
-        if (x$p_values[j] < 0.001)
-            stars[j] <- "***"
-        else if (x$p_values[j] < 0.01)
-            stars[j] <- "**"
-        else if (x$p_values[j] < 0.05)
-            stars[j] <- "*"
-        else if (x$p_values[j] < 0.1)
-            stars[j] <- "."
-        else
-            stars[j] <- " "
-    }
-    p.values <- paste(format(x$p_values, digits = digits),
-                      stars)
-    output <- data.frame(cbind(Estimate = coef,
-                               `Std. Error` = std.err,
-                               `t value` = t.stats,
-                               `Pr(>|t|)` = p.values),
-                         row.names = rows, check.names = FALSE)
-    print(format(output, justify = "left"))
-    
-    cat("---\n")
-    cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n\n")
     cat("R-squared:", x$r2, "\n")
     cat("Condition Number:", x$condition_no, "\n")
     
