@@ -9,7 +9,7 @@
 arraydb.to.arrayr <- function (str, type = "double", n = 1)
 {
     if (is.null(str)) return (NULL)
-    
+
     if (type == "character")
         res <- character(0)
     else if (type == "integer")
@@ -18,7 +18,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
         res <- logical(0)
     else
         res <- numeric(0)
-    
+
     for (i in seq(str))
     {
         if (is.na(str[i])) {
@@ -40,7 +40,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
             else
                 elm <- as.numeric(elm)
         }
-        
+
         res <- rbind(res, elm)
     }
     row.names(res) <- NULL
@@ -81,7 +81,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
         else {
             if (!.is.arg.string(distributed.by))
                 stop("distributed.by must be a string or NULL!")
-            if (distributed.by == "") # "" means no distributed by 
+            if (distributed.by == "") # "" means no distributed by
                 dist.str <- ""
             else
                 dist.str <- paste("DISTRIBUTED BY (", distributed.by, ")",
@@ -170,13 +170,23 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 
 .unique.string <- function ()
 {
-    ## r1 <- sample(seq_len(100000000), 1)
-    r1 <- as.integer(runif(1) * 1e8) + 1
-    r2 <- unclass(as.POSIXct(strptime(date(),"%c")))[1]
-    r3 <- r2 %% r1
-    paste("__madlib_temp_", r1, "_", r2, "_", r3, "__", sep = "")
+    hex_digits <- c(as.character(0:9), letters[1:6])
+    y_digits <- hex_digits[9:12]
+
+    s <- paste(
+        paste(sample(hex_digits, 8), collapse=''),
+        paste(sample(hex_digits, 4), collapse=''),
+        ## paste('4', sample(hex_digits, 3), collapse=''),
+        paste(sample(y_digits,1), sample(hex_digits, 3),
+              collapse='', sep = ''),
+        paste(sample(hex_digits, 12), collapse=''), sep='_')
+    s <- paste("__madlib_temp_", s, "__", sep = "")
+    s
 }
 
+.unique.pattern <- function()
+    "__madlib_temp_[a-f\\d]{8}_[a-f\\d]{4}_[a-f\\d]{6}_[a-f\\d]{12}__"
+
 ## -----------------------------------------------------------------------
 
 ## strip the leading and trailing white spaces
@@ -192,11 +202,15 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 .analyze.formula <- function (formula, data, fdata = data, refresh = FALSE,
                               is.factor = NA, cols = NA, suffix = NA,
                               grp.vars = NULL, grp.expr = NULL)
-{    
+{
     f.str <- strsplit(paste(deparse(formula), collapse = ""), "\\|")[[1]]
     ## f.str <- .replace.array(f.str, data)
     fstr <- f.str[1]
-    
+    fstr <- gsub("as\\.factor\\((((?!as\\.factor).)*)\\)", "factor(\\1)",
+                 fstr, perl = T)
+    fstr <- gsub("factor\\((((?!as\\.factor).)*)\\)", "as.factor(\\1)",
+                 fstr, perl = T)
+
     f2 <- f.str[2] # grouping columns, might be NA
 
     ## fdata <- .expand.array(fdata)
@@ -207,15 +221,12 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     } else if (!is.na(f2)) {
         f2.terms <- terms(formula(paste("~", f2)))
         f2.labels <- attr(f2.terms, "term.labels")
-        ## inter <- intersect(f2.labels, names(fdata))
-        ## if (length(inter) != length(f2.labels))
-        ##     stop("The grouping part of the formula is not quite right!")
+
         ## ## grouping column do not use factor
         f2.labels <- gsub("I\\((.*)\\)", "\\1", f2.labels, perl = T)
         f2.labels <- gsub("as.factor\\((.*)\\)", "\\1", f2.labels, perl = T)
         f2.labels <- gsub("factor\\((.*)\\)", "\\1", f2.labels, perl = T)
-        ## f2.labels <- .replace.with.quotes(f2.labels, data@.col.name)
-        ## ## f2.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f2.labels)
+
         grp.expr <- f2.labels
         for (i in seq_len(length(f2.labels))) {
             if (! f2.labels[i] %in% names(data)) {
@@ -223,7 +234,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
                 data[[grp.col]] <- eval(parse(text = paste("with(data, ",
                                               f2.labels[i], ")", sep = "")))
                 f2.labels[i] <- grp.col
-            }                
+            }
         }
         grp <- paste(f2.labels, collapse = ", ")
     } else {
@@ -243,7 +254,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     ## the 1st row is the dependent variable
     f.factors <- attr(f.terms, "factors")
     f.labels <- attr(f.terms, "term.labels") # each terms on the right side
-    ## f.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f.labels)    
+    ## f.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f.labels)
     right.hand <- paste(f.labels, collapse = "+")
     if (refresh) { # second pass
         replace.cols <- cols[is.factor]
@@ -255,7 +266,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
             col <- replace.cols[i]
             new.col <- names(data)[grep(paste(col, suffix[i], sep=""),
                                         names(data))]
-            if (identical(new.col, character(0))) 
+            if (identical(new.col, character(0)))
                 new.col <- "1"
             else
                 new.col <- paste("(", paste("`", new.col, "`",
@@ -266,17 +277,6 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
                                right.hand, perl = TRUE)
         }
     } else { # first pass
-        ## # make all text columns to be factor
-        ## for (i in seq_len(length(names(data)))) {
-        ##     if (data@.col.data_type[i] %in% .txt.types) {
-        ##         col <- data@.col.name[i]
-        ##         right.hand <- gsub(col,
-        ##                            paste0("as\\.factor\\(",
-        ##                                   col, "\\)"),
-        ##                            right.hand)
-        ##     }
-        ## }
-        
         ## find all the factor columns
         right.hand <- gsub("as.factor\\s*\\((.*)\\)",
                            "factor(\\1)", right.hand, perl = T)
@@ -286,8 +286,9 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
         col <- .strip(gsub("factor\\s*\\(([^\\(\\)]+)\\)", "\\1", elm,
                            perl = T))
         if (!all(col %in% names(data)))
-            stop("You can only make a existing column of",
-                 " the data into factor!")
+            stop("At least one of the variables cannot be set to be a factor ",
+                 "because either it does not exist in the data table ",
+                 "or it is an element of an array!")
 
         ## make sure the returned object is always db.Rquery
         for (i in seq_len(length(data@.col.name)))
@@ -296,13 +297,12 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
         ## factors added by formula
         for (cl in col) data[[cl]] <- as.factor(data[[cl]])
     }
-
+    
     right.hand <- gsub("as\\.factor\\((((?!as\\.factor).)*)\\)", "\\1", right.hand, perl = T)
     right.hand <- gsub("factor\\((((?!factor).)*)\\)", "\\1", right.hand, perl = T)
     f.terms1 <- terms(formula(paste("~", right.hand)), data = fake.data)
     f.labels <- attr(f.terms1, "term.labels")
     ## f.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f.labels)
-
     f.intercept <- attr(f.terms, "intercept")
     labels <- gsub("\\[(\\d+):(\\d+)\\]", "[\\1@\\2]", f.labels)
     labels <- gsub(":", "*", labels, perl = T) # replace interaction : with *
@@ -338,20 +338,18 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
                     paste(labels, collapse = ", "), "))",
                     sep = "")))
     a.labels <- unlist(sapply(a, function(x) x[,]@.expr))
-
     b <- eval(parse(text = paste("with(vdata, c(",
                     paste(setdiff(rownames(f.factors),
                                   colnames(f.factors)),
                           collapse = ", "), "))", sep = "")))
     b.labels <- unlist(sapply(b, function(x) x[,]@.expr))
-
     labels <- .strip(setdiff(a.labels, b.labels), "`")
-    
+
     ## labels <- .replace.with.quotes(labels, data@.col.name)
     ## remove grouping columns, when there is no intercept term
     if (!is.null(f2.labels) && f.intercept != 0)
         labels <- setdiff(labels, .strip(f2.labels, "`"))
-    
+
     ind.var <- paste("array[", intercept.str,
                      paste(labels, collapse = ","),
                      "]", sep = "") # independent variable
@@ -360,17 +358,52 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     ind.var <- .consistent.func(ind.var)
     labels <- .consistent.func(labels)
 
-    ##
-    ## dep.var <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", dep.var)
-    ## ind.var <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", ind.var)
-    ## grp <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", grp)
-    ## labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", labels)
     dep.var <- gsub("`", "", dep.var)
     ind.var <- gsub("`", "", ind.var)
     if (!is.null(grp)) grp <- gsub("`", "", grp)
     labels <- gsub("`", "", labels)
-    ##
-    
+
+    factor.full <- rep(FALSE, length(names(data)))
+    if (!refresh) {
+        model.vars <- .prepare.ind.vars(orig.labels)
+        model.vars <- gsub("`\"([^\\[\\]]*)\"\\[(\\d+)\\]`", "`\\1[\\2]`", model.vars)
+        vars <- unique(all.vars(parse(text = model.vars)))
+        for (var in vars) {
+            tmp <- eval(parse(text = paste("with(data, ", var, ")", sep = "")))
+            if (tmp@.col.data_type %in% c("boolean", .txt.types) &&
+                !tmp@.is.factor) {
+                data[[var]] <- as.factor(data[[var]])
+            }
+        }
+
+        ## re-analyze the formula
+        if (sum(data@.is.factor) > 0) {
+            l <- length(names(data))
+            fake.data <- as.data.frame(array(1, dim = c(3, l)))
+            names(fake.data) <- names(data)
+            us <- rep('', l)
+            for (i in seq_len(l)) {
+                if (data@.is.factor[i]) {
+                    us[i] <- paste(
+                        sample(c(as.character(0:9), letters[1:6]), 8),
+                        collapse = '')
+                    fake.data[,i] <- paste(us[i], c('a', 'b', 'c'), sep = '')
+                }
+            }
+            f1 <- formula(fstr)
+            term <- .modeling.formula(f1, fake.data)
+            ## factor.full <- rep(FALSE, l)
+            for (i in seq_len(l)) {
+                if (data@.is.factor[i]) {
+                    if (any(grepl(us[i]%+%'a', term)) &&
+                        any(grepl(us[i]%+%'b', term)) &&
+                        any(grepl(us[i]%+%'c', term)))
+                        factor.full[i] <- TRUE
+                }
+            }
+        }
+    }
+
     list(dep.str = dep.var, origin.dep = origin.dep,
          origin.ind = orig.labels,
          ind.str = ind.var,
@@ -378,11 +411,27 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
          ind.vars = labels,
          has.intercept = as.logical(f.intercept),
          data = data,
+         factor.full = factor.full,
          terms = f.terms)
 }
 
 ## -----------------------------------------------------------------------
 
+.modeling.formula <- function(formula, data)
+{
+    mf <- match.call(expand.dots = FALSE)
+    m <- match(c("formula", "data"), names(mf), 0L)
+    mf <- mf[c(1L, m)]
+    mf$drop.unused.levels <- TRUE
+    mf[[1L]] <- quote(stats::model.frame)
+    mf <- eval(mf, parent.frame())
+    mt <- attr(mf, "terms")
+    x <- model.matrix(mt, mf, contrasts)
+    colnames(x)
+}
+
+## ----------------------------------------------------------------------
+
 ## R's log is SQL's log(exp(1.), x)
 ## R's log10 is SQL's log
 .consistent.func <- function (strs)
@@ -394,6 +443,16 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     res
 }
 
+## ----------------------------------------------------------------------
+
+## reverse of the above function
+.reverse.consistent.func <- function (strs)
+{
+    res <- gsub("log\\s*\\(", "log10(", strs)
+    res <- gsub("ln\\s*\\(", "log(", res)
+    res
+}
+
 ## -----------------------------------------------------------------------
 
 .replace.with.quotes <- function (vars, cols)
@@ -444,13 +503,29 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     old.level
 }
 
+## ----------------------------------------------------------------------
+
+.db.data.frame2db.Rquery <- function(x)
+{
+    if (is(x, "db.data.frame")) {
+        if (length(names(x)) == 1 && x@.col.data_type == "array") {
+            if (array)
+                x <- db.array(x)
+            else
+                x <- x[[names(x)]]
+        } else
+            x <- x[,]
+    }
+    x
+}
+
 ## -----------------------------------------------------------------------
 
 ## If an independent variable is an array, it needs special treatment
 .is.array <- function (labels, data)
 {
     nlabels <- character(0)
-    data <- data[,]
+    data <- .db.data.frame2db.Rquery(data)
     if (data@.parent == data@.source)
         tbl <- data@.parent
     else
@@ -458,7 +533,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     if (data@.where != "") where.str <- paste(" where", data@.where)
     else where.str <- ""
     conn.id <- conn.id(data)
-    
+
     for (i in seq_len(length(labels))) {
         if (labels[i] %in% names(data) &&
             data@.col.data_type[i] == "array") {
@@ -493,7 +568,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     if (data@.where != "") where.str <- paste(" where", data@.where)
     else where.str <- ""
     conn.id <- conn.id(data)
-    
+
     for (i in seq_len(length(data@.col.name))) {
         if (data@.col.data_type[i] == "array") {
             n <- .db.getQuery(paste("select array_upper(\"",
@@ -530,7 +605,7 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
                (s == where1 && t != where2)) {
         return (FALSE)
     }
-    
+
     s1 <- gsub("^\\((.*)\\) (and|or) \\((.*)\\)$", "\\1", s, perl = TRUE)
     if (s1 == s) return (s == t)
     s2 <- gsub("^\\((.*)\\) (and|or) \\((.*)\\)$", "\\3", s, perl = TRUE)
